# Generated by Django 5.1.6 on 2025-03-28 10:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('history4feed', '0002_feed_freshness_alter_feed_feed_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='feed',
            name='description',
            field=models.CharField(default=None, help_text='found in the <channel> of RSS output. Is always kept up to date with the latest feed import values for this property.', max_length=10240, null=True),
        ),
    ]
